<?php

namespace Drupal\dsfr_twig_components\Twig;

class Resources {

  /**
   * DSFR version supported by the module
   */
  public static function version(): string { return '1.12.1'; }

  /**
   * Provides a list of operational Twig components (DSFR)
   */
  public static function functionsReady(): array {

    $d_comp = 'composants-et-modeles/composants/';

    return [
      self::buildItem('accordion', $d_comp . 'accordeon', 'edition'),
      self::buildItem('alert', $d_comp . 'alerte', 'alert'),
      self::buildItem('badge', $d_comp . 'badge', 'alert' ),
      self::buildItem('button', $d_comp . 'bouton', 'form' ),
      self::buildItem('callout', $d_comp . 'mise-en-avant', 'alert' ),
      self::buildItem('card', $d_comp . 'carte', 'collection' ),
      self::buildItem('france-connect', $d_comp . 'bouton-franceconnect', 'form' ),
      self::buildItem('highlight', $d_comp . 'mise-en-exergue', 'edition' ),
      self::buildItem('image', $d_comp . '/contenu-medias', 'edition' ),
      //self::buildItem('modal', $d_comp . '/modale', 'edition' ),
      self::buildItem('notice', $d_comp . '', 'alert' ),
      self::buildItem('link', $d_comp . 'lien', 'edition' ),
      self::buildItem('quote', $d_comp . 'citation', 'edition' ),
      self::buildItem('search', $d_comp . 'barre-de-recherche', 'form' ),
      self::buildItem('tabs', $d_comp . 'onglet', 'edition' ),
      self::buildItem('tag', $d_comp . 'tag', 'other' ),
      self::buildItem('tile', $d_comp . 'tuile', 'collection' ),
      self::buildItem('title', 'fondamentaux/typographie', 'edition','dsfr', false ),
      //self::buildItem('toggle', $d_comp . 'interrupteur', 'form' ),
      self::buildItem('tooltip', $d_comp . 'infobulle', 'alert' ),
    ];
  }

  /**
   * Single line of available colors
   */
  public static function colorLine(): string { 
    return 'blue-ecume, blue-cumulus, green-archipel, green-menthe, green-bourgeon, green-tilleul-verveine, 
    yellow-tournesol, yellow-moutarde,brown-cafe-creme, brown-caramel, brown-opera, 
    pink-macaron, pink-tuile, purple-glycine, beige-gris-galet.'; 
  }

  /**
   * See: https://www.systeme-de-design.gouv.fr/fondamentaux/icone
   * Please update the icons in this PHP table rather than in its Json equivalent.
   */
  public static function iconsType(): array {

    $d = ['author' => 'dsfr'];
    $r = ['author' => 'remixicon'];

    return [

      'building' => [
        'ancient-gate-fill' => [],
        'ancient-gate-line' => [],
        'ancient-pavilion-fill' => [],
        'ancient-pavilion-line' => [],
        'bank-fill' => [],
        'bank-line' => [],
        'building-fill' => [],
        'building-line' => [],
        'community-fill' => [],
        'community-line' => [],
        'government-fill' => [],
        'government-line' => [],
        'home-4-fill' => [],
        'home-4-line' => [],
        'hospital-fill' => [],
        'hospital-line' => [],
        'hotel-fill' => [],
        'hotel-line' => [],
        'store-fill' => [],
        'store-line' => []
      ],

      'business' => [
        'archive-fill' => [],
        'archive-line' => [],
        'attachment-fill' => [],
        'attachment-line' => [],
        'award-fill' => [],
        'award-line' => [],
        'bar-chart-box-fill' => [],
        'bar-chart-box-line' => [],
        'bookmark-fill' => [],
        'bookmark-line' => [],
        'briefcase-fill' => [],
        'briefcase-line' => [],
        'calendar-2-fill' => [],
        'calendar-2-line' => [],
        'calendar-event-fill' => [],
        'calendar-event-line' => [],
        'calendar-fill' => [],
        'calendar-line' => [],
        'cloud-fill' => [],
        'cloud-line' => [],
        'copyright-fill' => [],
        'copyright-line' => [],
        'customer-service-fill' => [],
        'customer-service-line' => [],
        'flag-fill' => [],
        'flag-line' => [],
        'global-fill' => [],
        'global-line' => [],
        'line-chart-fill' => [],
        'line-chart-line' => [],
        'links-fill' => [],
        'links-line' => [],
        'mail-fill' => [],
        'mail-line' => [],
        'mail-open-fill' => [],
        'mail-open-line' => [],
        'medal-fill' => [],
        'medal-line' => [],
        'pie-chart-2-fill' => [],
        'pie-chart-2-line' => [],
        'pie-chart-box-fill' => [],
        'pie-chart-box-line' => [],
        'printer-fill' => [],
        'printer-line' => [],
        'profil-fill' => [],
        'profil-line' => [],
        'projector-2-fill' => [],
        'projector-2-line' => [],
        'send-plane-fill' => [],
        'send-plane-line' => [],
        'slideshow-fill' => [],
        'slideshow-line' => [],
        'window-fill' => [],
        'window-line' => []     
      ],

      'communication' => [
        'chat-2-fill' => [],
        'chat-2-line' => [],
        'chat-3-fill' => [],
        'chat-3-line' => [],
        'chat-check-fill' => [],
        'chat-check-line' => [],
        'chat-delete-fill' => [],
        'chat-delete-line' => [],
        'chat-poll-fill' => [],
        'chat-poll-line' => [],
        'discuss-fill' => [],
        'discuss-line' => [],
        'feedback-fill' => [],
        'feedback-line' => [],
        'message-2-fill' => [],
        'message-2-line' => [],
        'question-answer-fill' => [],
        'question-answer-line' => [],
        'questionnaire-fill' => [],
        'questionnaire-line' => [],
        'video-chat-fill' => [],
        'video-chat-line' => []
      ],
      
      'design' => [
        'ball-pen-fill' => [],
        'ball-pen-line' => [],
        'brush-3-fill' => [],
        'brush-3-line' => [],
        'brush-fill' => [],
        'brush-line' => [],
        'contrast-fill' => [],
        'contrast-line' => [],
        'crop-fill' => [],
        'crop-line' => [],
        'drag-move-2-fill' => [],
        'drag-move-2-line' => [],
        'drop-fill' => [],
        'drop-line' => [],
        'edit-box-fill' => [],
        'edit-box-line' => [],
        'edit-fill' => [],
        'edit-line' => [],
        'ink-bottle-fill' => [],
        'ink-bottle-line' => [],
        'layout-grid-fill' => [],
        'layout-grid-line' => [],
        'mark-pen-fill' => [],
        'mark-pen-line' => [],
        'paint-brush-fill' => [],
        'paint-brush-line' => [],
        'paint-fill' => [],
        'paint-line' => [],
        'palette-fill' => [],
        'palette-line' => [],
        'pantone-fill' => [],
        'pantone-line' => [],
        'pen-nib-fill' => [],
        'pen-nib-line' => [],
        'pencil-fill' => [],
        'pencil-line' => [],
        'pencil-ruler-fill' => [],
        'pencil-ruler-line' => [],
        'sip-fill' => [],
        'sip-line' => [],
        'shapes-fill' => [],
        'shapes-line' => [],
        'table-fill' => [],
        'table-line' => []
      ],

      'development' => [
        'bug-fill' => [],
        'bug-line' => [],
        'code-box-fill' => [],
        'code-box-line' => [],
        'code-s-slash-line' => [],
        'cursor-fill' => [],
        'cursor-line' => [],
        'git-branch-fill' => [],
        'git-branch-line' => [],
        'git-commit-fill' => [],
        'git-commit-line' => [],
        'git-merge-fill' => [],
        'git-merge-line' => [],
        'git-pull-request-fill' => [],
        'git-pull-request-line' => [],
        'git-repository-commits-fill' => [],
        'git-repository-commits-line' => [],
        'git-repository-fill' => [],
        'git-repository-line' => [],
        'git-repository-private-fill' => [],
        'git-repository-private-line' => [],
        'terminal-box-fill' => [],
        'terminal-box-line' => [],
        'terminal-line' => [],
        'terminal-window-fill' => [],
        'terminal-window-line' => []    
      ],

      'device' => [
        'bluetooth-fill' => [],
        'bluetooth-line' => [],
        'computer-fill' => [],
        'computer-line' => [],
        'dashboard-3-fill' => [],
        'dashboard-3-line' => [],
        'database-fill' => [],
        'database-line' => [],
        'device-fill' => [],
        'device-line' => [],
        'hard-drive-2-fill' => [],
        'hard-drive-2-line' => [],
        'mac-fill' => [],
        'mac-line' => [],
        'phone-fill' => [],
        'phone-line' => [],
        'qr-code-fill' => [],
        'qr-code-line' => [],
        'rss-fill' => [],
        'rss-line' => [],
        'save-3-fill' => [],
        'save-3-line' => [],
        'save-fill' => [],
        'save-line' => [],
        'server-fill' => [],
        'server-line' => [],
        'smartphone-fill' => [],
        'smartphone-line' => [],
        'tablet-fill' => [],
        'tablet-line' => [],
        'tv-fill' => [],
        'tv-line' => [],
        'wifi-fill' => [],
        'wifi-line' => []
      ],

      'document' => [
        'article-fill' => [],
        'article-line' => [],
        'book-2-fill' => [],
        'book-2-line' => [],
        'booklet-fill' => [],
        'booklet-line' => [],
        'clipboard-fill' => [],
        'clipboard-line' => [],
        'draft-fill' => [],
        'draft-line' => [],
        'file-add-fill' => [],
        'file-add-line' => [],
        'file-download-fill' => [],
        'file-download-line' => [],
        'file-fill' => [],
        'file-line' => [],
        'file-pdf-fill' => [],
        'file-pdf-line' => [],
        'file-text-fill' => [],
        'file-text-line' => [],
        'folder-2-fill' => [],
        'folder-2-line' => [],
        'newspaper-fill' => [],
        'newspaper-line' => [],
        'survey-fill' => [],
        'survey-line' => [],
        'todo-fill' => [],
        'todo-line' => []
      ],
      
      'editor' => [
        'bold' => $d,
        'highlight' => $d,
        'quote-fill' => $d,
        'quote-line' => $d,        
        'code-view' => $r,
        'font-size' => $r,
        'h-1' => $r,
        'h-2' => $r,
        'h-3' => $r,
        'h-4' => $r,
        'h-5' => $r,
        'h-6' => $r,
        'hashtag' => $r,
        'italic' => $r,
        'link-unlink' => $r,
        'link' => $r,
        'list-ordered' => $r,
        'list-unordered' => $r,
        'question-mark' => $r,
        'separator' => $r,
        'space' => $r,
        'subscript' => $r,
        'superscript' => $r,
        'table-2' => $r,
        'translate-2' => $r,
      ],

      'finance' => [
        'bank-card-fill' => ['type' => 'finance'],
        'bank-card-line' => ['type' => 'finance'],
        'coin-fill' => ['type' => 'finance'],
        'gift-fill' => ['type' => 'finance'],
        'gift-line' => ['type' => 'finance'],
        'money-euro-box-fill' => ['type' => 'finance'],
        'money-euro-box-line' => ['type' => 'finance'],
        'money-euro-circle-fill' => ['type' => 'finance'],
        'money-euro-circle-line' => ['type' => 'finance'],
        'secure-payment-fill' => ['type' => 'finance'],
        'secure-payment-line' => ['type' => 'finance'],
        'shopping-bag-fill' => ['type' => 'finance'],
        'shopping-bag-line' => ['type' => 'finance'],
        'shopping-cart-2-fill' => ['type' => 'finance'],
        'shopping-cart-2-line' => ['type' => 'finance'],
        'trophy-fill' => ['type' => 'finance'],
        'trophy-line' => ['type' => 'finance'],
      ],
      
      'health' => [
        'capsule-fill' => [],
        'capsule-line' => [],
        'dislike-fill' => [],
        'dislike-line' => [],
        'dossier-fill' => [],
        'dossier-line' => [],
        'first-aid-kit-fill' => [],
        'first-aid-kit-line' => [],
        'hand-sanitizer-fill' => [],
        'hand-sanitizer-line' => [],
        'health-book-fill' => [],
        'health-book-line' => [],
        'heart-fill' => [],
        'heart-line' => [],
        'heart-pulse-fill' => [],
        'heart-pulse-line' => [],
        'lungs-fill' => [],
        'lungs-line' => [],
        'medicine-bottle-fill' => [],
        'medicine-bottle-line' => [],
        'mental-health-fill' => [],
        'mental-health-line' => [],
        'microscope-fill' => [],
        'microscope-line' => [],
        'psychotherapy-fill' => [],
        'psychotherapy-line' => [],
        'pulse-line' => [],
        'stethoscope-fill' => [],
        'stethoscope-line' => [],
        'surgical-mask-fill' => [],
        'surgical-mask-line' => [],
        'syringe-fill' => [],
        'syringe-line' => [],
        'test-tube-fill' => [],
        'test-tube-line' => [],
        'thermometer-fill' => [],
        'thermometer-line' => [],
        'virus-fill' => [],
        'virus-line' => [],
      ],
      
      'logos' => [
        'dailymotion-fill' => $d,
        'dailymotion-line' => $d,
        'tiktok-fill' => $d,
        'tiktok-line' => $d,
        'chrome-fill' => $r,
        'chrome-line' => $r,
        'edge-fill' => $r,
        'edge-line' => $r,
        'facebook-circle-fill' => $r,
        'facebook-circle-line' => $r,
        'firefox-fill' => $r,
        'firefox-line' => $r,
        'github-fill' => $r,
        'github-line' => $r,
        'google-fill' => $r,
        'google-line' => $r,
        'ie-fill' => $r,
        'ie-line' => $r,
        'instagram-fill' => $r,
        'instagram-line' => $r,
        'linkedin-box-fill' => $r,
        'linkedin-box-line' => $r,
        'mastodon-fill' => $r,
        'mastodon-line' => $r,
        'npmjs-fill' => $r,
        'npmjs-line' => $r,
        'remixicon-fill' => $r,
        'remixicon-line' => $r,
        'safari-fill' => $r,
        'safari-line' => $r,
        'slack-fill' => $r,
        'slack-line' => $r,
        'snapchat-fill' => $r,
        'snapchat-line' => $r,
        'telegram-fill' => $r,
        'telegram-line' => $r,
        'twitch-fill' => $r,
        'twitch-line' => $r,
        'twitter-x-fill' => $r,
        'twitter-x-line' => $r,
        'threads-fill' => $r,
        'threads-line' => $r,
        'twitter-fill' => $r,
        'twitter-line' => $r,
        'vimeo-fill' => $r,
        'vimeo-line' => $r,
        'vuejs-fill' => $r,
        'vuejs-line' => $r,
        'whatsapp-fill' => $r,
        'whatsapp-line' => $r,
        'youtube-fill' => $r,
        'youtube-line' => $r,      
      ],
      
      'map' => [
        'anchor-fill' => [],
        'anchor-line' => [],
        'bike-fill' => [],
        'bike-line' => [],
        'bus-fill' => [],
        'bus-line' => [],
        'car-fill' => [],
        'car-line' => [],
        'caravan-fill' => [],
        'caravan-line' => [],
        'charging-pile-2-fill' => [],
        'charging-pile-2-line' => [],
        'compass-3-fill' => [],
        'compass-3-line' => [],
        'cup-fill' => [],
        'cup-line' => [],
        'earth-fill' => [],
        'earth-line' => [],
        'france-fill' => [],
        'france-line' => [],
        'gas-station-fill' => [],
        'gas-station-line' => [],
        'goblet-fill' => [],
        'goblet-line' => [],
        'map-pin-2-fill' => [],
        'map-pin-2-line' => [],
        'map-pin-user-fill' => [],
        'map-pin-user-line' => [],
        'motorbike-fill' => [],
        'motorbike-line' => [],
        'passport-fill' => [],
        'passport-line' => [],
        'restaurant-fill' => [],
        'restaurant-line' => [],
        'road-map-fill' => [],
        'road-map-line' => [],
        'sailboat-fill' => [],
        'sailboat-line' => [],
        'ship-2-fill' => [],
        'ship-2-line' => [],
        'signal-tower-fill' => [],
        'signal-tower-line' => [],
        'suitcase-2-fill' => [],
        'suitcase-2-line' => [],
        'taxi-fill' => [],
        'taxi-line' => [],
        'train-fill' => [],
        'train-line' => [],
      ],
      
      'media' => [
        'camera-fill' => [],
        'camera-line' => [],
        'clapperboard-fill' => [],
        'clapperboard-line' => [],
        'equalizer-fill' => [],
        'equalizer-line' => [],
        'film-fill' => [],
        'film-line' => [],
        'gallery-fill' => [],
        'gallery-line' => [],
        'headphone-fill' => [],
        'headphone-line' => [],
        'image-add-fill' => [],
        'image-add-line' => [],
        'image-edit-fill' => [],
        'image-edit-line' => [],
        'image-fill' => [],
        'image-line' => [],
        'live-fill' => [],
        'live-line' => [],
        'mic-fill' => [],
        'mic-line' => [],
        'music-2-fill' => [],
        'music-2-line' => [],
        'notification-3-fill' => [],
        'notification-3-line' => [],
        'pause-circle-fill' => [],
        'pause-circle-line' => [],
        'play-circle-fill' => [],
        'play-circle-line' => [],
        'stop-circle-fill' => [],
        'stop-circle-line' => [],
        'volume-down-fill' => [],
        'volume-down-line' => [],
        'volume-mute-fill' => [],
        'volume-mute-line' => [],
        'volume-up-fill' => [],
        'volume-up-line' => [],
      ],
      
      'other' => [
        'leaf-fill' => [],
        'leaf-line' => [],
        'lightbulb-fill' => [],
        'lightbulb-line' => [],
        'plant-fill' => [],
        'plant-line' => [],
        'recycle-fill' => [],
        'recycle-line' => [],
        'scales-3-fill' => [],
        'scales-3-line' => [],
        'seedling-fill' => [],
        'seedling-line' => [],
        'umbrella-fill' => [],
        'umbrella-line' => [],
        'accessibility-fill' => [],
        'accessibility-line' => [],
        'ear-off-fill' => [],
        'ear-off-line' => [],
        'mental-disabilities-fill' => [],
        'mental-disabilities-line' => [],
        'sign-language-fill' => [],
        'sign-language-line' => [],
        'wheelchair-fill' => [],
        'wheelchair-line' => [],
      ],
      
      'system' => [
        'arrow-left-s-fill' => $d,
        'arrow-left-s-line' => $d,
        'arrow-right-s-fill' => $d,
        'arrow-right-s-line' => $d,
        'error-fill' => $d,
        'error-line' => $d,
        'info-fill' => $d,
        'info-line' => $d,
        'success-fill' => $d,
        'success-line' => $d,
        'warning-fill' => $d,
        'warning-line' => $d,
        'theme-fill' => $d,
        'add-circle-fill' => $r,
        'add-circle-line' => $r,
        'add-line' => $r,
        'alarm-warning-fill' => $r,
        'alarm-warning-line' => $r,
        'alert-fill' => $r,
        'alert-line' => $r,
        'arrow-down-fill' => $r,
        'arrow-down-line' => $r,
        'arrow-down-s-fill' => $r,
        'arrow-down-s-line' => $r,
        'arrow-go-back-fill' => $r,
        'arrow-go-back-line' => $r,
        'arrow-go-forward-fill' => $r,
        'arrow-go-forward-line' => $r,
        'arrow-left-fill' => $r,
        'arrow-left-line' => $r,
        'arrow-right-fill' => $r,
        'arrow-right-line' => $r,
        'arrow-right-up-line' => $r,
        'arrow-up-fill' => $r,
        'arrow-up-line' => $r,
        'arrow-up-s-fill' => $r,
        'arrow-up-s-line' => $r,
        'check-line' => $r,
        'checkbox-circle-fill' => $r,
        'checkbox-circle-line' => $r,
        'checkbox-fill' => $r,
        'checkbox-line' => $r,
        'close-circle-fill' => $r,
        'close-circle-line' => $r,
        'close-line' => $r,
        'delete-fill' => $r,
        'delete-line' => $r,
        'download-fill' => $r,
        'download-line' => $r,
        'error-warning-fill' => $r,
        'error-warning-line' => $r,
        'external-link-fill' => $r,
        'external-link-line' => $r,
        'eye-fill' => $r,
        'eye-line' => $r,
        'eye-off-fill' => $r,
        'eye-off-line' => $r,
        'filter-fill' => $r,
        'filter-line' => $r,
        'arrow-left-s-first-line' => $r,
        'arrow-left-s-line-double' => $r,
        'arrow-right-s-last-line' => $r,
        'arrow-right-s-line-double' => $r,
        'information-fill' => $r,
        'information-line' => $r,
        'lock-fill' => $r,
        'lock-line' => $r,
        'lock-unlock-fill' => $r,
        'lock-unlock-line' => $r,
        'logout-box-r-fill' => $r,
        'logout-box-r-line' => $r,
        'menu-2-fill' => $r,
        'menu-fill' => $r,
        'more-fill' => $r,
        'more-line' => $r,
        'notification-badge-fill' => $r,
        'notification-badge-line' => $r,
        'question-fill' => $r,
        'question-line' => $r,
        'refresh-fill' => $r,
        'refresh-line' => $r,
        'search-fill' => $r,
        'search-line' => $r,
        'settings-5-fill' => $r,
        'settings-5-line' => $r,
        'shield-fill' => $r,
        'shield-line' => $r,
        'sort-asc-desc-line' => $r,
        'sort-asc-desc-fill' => $r,
        'sort-desc-line' => $r,
        'star-fill' => $r,
        'star-line' => $r,
        'star-s-fill' => $r,
        'star-s-line' => $r,
        'subtract-line' => $r,
        'thumb-down-fill' => $r,
        'thumb-down-line' => $r,
        'thumb-up-fill' => $r,
        'thumb-up-line' => $r,
        'time-fill' => $r,
        'time-line' => $r,
        'timer-fill' => $r,
        'timer-line' => $r,
        'upload-2-fill' => $r,
        'upload-2-line' => $r,
        'upload-fill' => $r,
        'upload-line' => $r,
        'zoom-in-fill' => $r,
        'zoom-in-line' => $r,
        'zoom-out-fill' => $r,
        'zoom-out-line' => $r,      
      ],
      
      'user' => [
        'account-circle-fill' => [],
        'account-circle-line' => [],
        'account-fill' => [],
        'account-line' => [],
        'account-pin-circle-fill' => [],
        'account-pin-circle-line' => [],
        'admin-fill' => [],
        'admin-line' => [],
        'group-fill' => [],
        'group-line' => [],
        'parent-fill' => [],
        'parent-line' => [],
        'team-fill' => [],
        'team-line' => [],
        'user-add-fill' => [],
        'user-add-line' => [],
        'user-fill' => [],
        'user-line' => [],
        'user-heart-fill' => [],
        'user-heart-line' => [],
        'user-search-fill' => [],
        'user-search-line' => [],
        'user-setting-fill' => [],
        'user-setting-line' => [],
        'user-star-fill' => [],
        'user-star-line' => [],
      ],
      
      'weather' => [
        'cloudy-2-fill' => [],
        'cloudy-2-line' => [],
        'flashlight-fill' => [],
        'flashlight-line' => [],
        'moon-fill' => [],
        'moon-line' => [],
        'fire-line' => [],
        'fire-fill' => [],
        'flood-line' => [],
        'flood-fill' => [],
        'avalanches-fill' => [],
        'submersion-fill' => [],
        'heavy-showers-fill' => [],
        'heavy-showers-line' => [],
        'snowy-fill' => [],
        'snowy-line' => [],
        'sparkling-2-fill' => [],
        'sparkling-2-line' => [],
        'temp-cold-fill' => [],
        'temp-cold-line' => [],
        'tornado-fill' => [],
        'tornado-line' => [],
        'typhoon-fill' => [],
        'typhoon-line' => [],
        'windy-fill' => [],
        'windy-line' => [],
      ] 

    ];
  }


  /**
   * See: https://www.systeme-de-design.gouv.fr/fondamentaux/pictogramme
   * Please update the icons in this PHP table rather than in its Json equivalent.
   */
  public static function pictogramsType(): array {

    return [
      'buildings' => [
        'city-hall' => [],
        'factory' => [],
        'house' => [],
        'nuclear-plant' => [],
        'school' => [],
      ],
      'digital' => [
        'application' => [],
        'avatar' => [],
        'calendar' => [],
        'coding' => [],
        'data-visualization' => [],
        'internet' => [],
        'mail-send' => [],
        'search' => [],
      ],
      'document' => [
        'contract' => [],
        'document-add' => [],
        'document-download' => [],
        'document-signature' => [],
        'document' => [],
        'driving-licence' => [],
        'national-identity-card' => [],
        'passport' => [],
        'tax-stamp' => [],
        'vehicle-registration' => [],
      ],
      'environment' => [
        'environment' => [],
        'food' => [],
        'grocery' => [],
        'human-cooperation' => [],
        'leaf' => [],
        'moon' => [],
        'mountain' => [],
        'sun' => [],
        'tree' => [],
      ],
      'health' => [
        'health' => [],
        'hospital' => [],
        'vaccine' => [],
        'virus' => [],
      ],
      'institutions' => [
        'firefighter' => [],
        'gendarmerie' => [],
        'justice' => [],
        'money' => [],
        'police' => [],
      ],
      'leisure' => [
        'book' => [],
        'community' => [],
        'culture' => [],
        'digital-art' => [],
        'paint' => []
      ],
      'map' => [
        'airport' => [],
        'location-france' => [],
        'luggage' => [],
        'map' => [],
      ],
      'system' => [
        'connection-lost' => [],
        'error' => [],
        'information' => [],
        'notification' => [],
        'padlock' => [],
        'success' => [],
        'system' => [],
        'technical-error' => [],
        'warning' => [],
      ]
    ];
  }

  /**
   * Method used by the list of operational components
   */
  public static function buildItem( 
    string $name,
    string $url, 
    string $parent, 
    string $type = 'dsfr', 
    bool $official = true ): array {

    return [ 
      'name'     => $name,
      'url'      => $url,
      'parent'   => $parent, 
      'type'     => $type, 
      'official' => $official
    ];
  }

  /**
   * Provides a Json file containing the icon table
   */
  public static function iconsJson( string $directory ): array {

    return self::convertJson( self::iconsType(), $directory );
  }

  /**
   * Provides a Json file containing the pictogram table
   */
  public static function pictogramsJson( string $directory ): array {

    return self::convertJson( self::pictogramsType(), $directory, 'pictograms' );
  }

  /**
   * Provides a Json file
   */
  public static function convertJson( 
    array $items_type_full,
    string $directory, 
    string $label = 'icons'
  ): array {

    $data = [];
    foreach( $items_type_full as $key => $get_items ) {
      foreach( $get_items as $item => $infos ) {
        $data[$key][] = $item;
      }
    }

    $json = json_encode($data, JSON_PRETTY_PRINT);
    $directory = __DIR__ . $directory;

    $status = 'error';
    if (!is_dir($directory)) { 
      mkdir( $directory );
      mkdir( $directory );
    }

    if (!is_writable( $directory )) {
      
      $msg = $directory . " is not writable.";

    } else {

      file_put_contents( $directory . $label . '.json', $json);
      $msg = 'File generated!';
      $status = 'success';
    }

    return [$msg, $status];
  }
}